#include <cstdlib>
#include <cstdio>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <netdb.h>

#include <list>
#include <algorithm>

#define MAX_LENGTH	20480

int server_socket = 0;
int my_id = 0;
int my_capacity = 0;
int my_money = 0;
int my_points = 0;
char line[MAX_LENGTH];

typedef struct board_t
{
	int width;
	int height;
	char** value;
} BOARD;

typedef struct robot_t
{
	int id;
	int x;
	int y;
	int packages;
	int pushed;
} ROBOT;

typedef struct package_t
{
	int id;
	int x;
	int y;
	int weight;
} PACKAGE;

typedef struct position_t
{
	int x;
	int y;
} POSITION;

list<PACKAGE*> my_packages;
list<PACKAGE*> packages;
ROBOT* my_robot;
list<ROBOT*> robots;
BOARD game_board;
list<char> directions;
POSITION my_destination;

void PrintStatus();
void PrintRobot(ROBOT*&);
void PrintPackage(PACKAGE*&);
void PrintBoard(BOARD&);

int Connect(char* server_name, int server_port);
void CheckRobots(char*);
void CheckPackages(char*);

int Distance(POSITION p1, POSITION p2) {return (abs(p1.x-p2.x)+abs(p1.y-p2.y));}
void SolveMaze(int, int, int, int);
void MakeUpYourMind();
POSITION GetClosestBase();
POSITION GetClosestRobot();
POSITION GetClosestPackageRobot();
POSITION GetClosestPackageDropZone();
POSITION GetHeaviestPackageDropZone();
bool SomethingToDrop();

int NumberOfWalls(POSITION);
int NumberOfWater(POSITION);

int main(int argc, char** argv)
{
	memset(&my_robot, 0, sizeof(ROBOT));
	memset(&game_board, 0, sizeof(BOARD));

	printf("Welcome to B3nd3r 1.0\n");

	if (argc != 3)
	{
		printf("Usage: ./runme server port\n");
		return -1;
	}

	char* server_name = argv[1];
	int server_port = atoi(argv[2]);

	//	Initialization
	printf("Initialization...\n");

	printf("Connecting to %s at port %i...\n", server_name, server_port);

	if (Connect(server_name, server_port) != 0)
	{
		printf("Could not connect to server...ABORTING\n");
		return -2;
	}

	printf("Waiting for game board\n");
	int received = recv(server_socket, line, MAX_LENGTH, 0);
	line[received] = 0;

	int characters = 0;

	printf("I received %i bytes\n", received);

	char* short_line = strtok(line, "\n");
	characters += strlen(short_line)+1;

	sscanf(short_line, "%i %i", &game_board.width, &game_board.height);

	printf("width: %i\nheight: %i\n", game_board.width, game_board.height);

	game_board.value = new char*[game_board.height];

	for (int i = 0; i < game_board.height; i++)
	{
		if (received-characters < game_board.width)
			received += recv(server_socket, line+received, MAX_LENGTH-received, 0);

		short_line = strtok(NULL, "\n");
		characters += strlen(short_line)+1;

		game_board.value[i] = new char[strlen(short_line)+1];
		strcpy(game_board.value[i], short_line);
	}

	short_line = strtok(NULL, "\n");
	characters += strlen(short_line)+1;
	sscanf(short_line, "%i %i %i", &my_id, &my_capacity, &my_money);

	char* robot_list = strtok(NULL, "\n");
	characters += strlen(robot_list)+1;

	if (received-characters == 0)
		received += recv(server_socket, line+received, MAX_LENGTH-received, 0);
	char* package_list = strtok(NULL, "\n");

	my_robot = new ROBOT();
	my_robot->id = my_id;
	my_robot->packages = 0;
	my_robot->x = -1;
	my_robot->y = -1;
	robots.push_front(my_robot);
	printf("Game begins... ;-)\n");

	while (true)
	{
		CheckRobots(robot_list);
		CheckPackages(package_list);

		PrintBoard(game_board);
		PrintStatus();

		MakeUpYourMind();

		int received = recv(server_socket, line, MAX_LENGTH, 0);
		if (received == 0)
			break;
		line[received] = 0;

		robot_list = strtok(line, "\n");
		package_list = strtok(NULL, "\n");

		if (robot_list && strncmp(robot_list, "Robot", 5) == 0)
			break;
		if (package_list && strncmp(package_list, "Robot", 5) == 0)
			break;

		printf("****************END OF TURN*****************\n");
	}

	PrintBoard(game_board);
	PrintStatus();

	printf("%s\n", line);

	printf("GAME OVER\n");

	return 0;
}

int Connect(char* server_name, int server_port)
{
	char server_ip[20];
	struct hostent *info;
	info = gethostbyname(server_name);
	if (!info)
	{
		printf("try using a real server...\n");
		return -1;
	}
	strcpy(server_ip, inet_ntoa(*(struct in_addr *)*(info->h_addr_list)));

	struct sockaddr_in addresse;

	server_socket = socket(AF_INET, SOCK_STREAM, 0);

	addresse.sin_family = AF_INET;
	addresse.sin_addr.s_addr = inet_addr(server_ip);
	addresse.sin_port = htons(server_port);

	int result = connect(server_socket, (struct sockaddr *)&addresse, sizeof(addresse));

	if (result == 0)
	{
		printf("Sending my name\n");
		unsigned int sent = send(server_socket, "Player\n", strlen("Player\n"), 0);
		if (sent != strlen("Player\n"))
			result = -1;
	}

	return result;
}

void PrintStatus()
{
	printf("board:\twidth\theight\n");
	printf("\t%i\t%i\n", game_board.width, game_board.height);

	printf("Name\tPacks\tPosition\tDistance\tSpace\tMoney\tPoints\tDestination:\n");

	POSITION my_position = {my_robot->x, my_robot->y};
	printf("%i\t%i\t(%3i,%3i)\t%i\t%i\t%i\t%i\t(%3i,%3i)\n", my_id, my_robot->packages, my_robot->x, my_robot->y, Distance(my_position, my_destination), my_capacity, my_money, my_points, my_destination.x, my_destination.y);

	list<ROBOT*>::iterator iterator;
	if (robots.size())
		for (iterator = robots.begin(); iterator != robots.end(); ++iterator)
			if ((*iterator)->id != my_robot->id)
			{
				POSITION robot_position = {(*iterator)->x, (*iterator)->y};
				printf("%i\t%i\t(%3i,%3i)\t%i\n", (*iterator)->id, (*iterator)->packages, robot_position.x, robot_position.y, Distance(my_position, robot_position));
			}
}

void PrintRobot(ROBOT*& robot)
{
	printf("robot id: %i (%i,%i)\n", robot->id, robot->x, robot->y);
}

void PrintPackage(PACKAGE*& package)
{
	printf("package id: %i\t(%i,%i)\tweight %i\n", package->id, package->x, package->y, package->weight);
}

void PrintBoard(BOARD& board)
{
	list<ROBOT*>::iterator iterator;
	for (int i = board.height-1; i >= 0; i--)
	{
		char* temp = new char[board.width];
		strcpy(temp, board.value[i]);

		if (robots.size())
		{
			for (iterator = robots.begin(); iterator != robots.end(); ++iterator)
			{
				if (i == (*iterator)->y-1)
				{
					if ((*iterator)->id == my_robot->id)
						temp[(*iterator)->x-1] = 'W';
					else
						temp[(*iterator)->x-1] = 'X';
				}
			}
		}

		printf("%s\n", temp);
		delete [] temp;
	}
}

void CheckRobots(char* robot_list)
{
	printf("robot_list: %s\n", robot_list);

	list<ROBOT*> new_robots;
	list<char*> items;
	char* item = strtok(robot_list, "#");
	while (item)
	{
		items.push_back(item);
		item = strtok(NULL, "#");
	}

	while (!items.empty())
	{
		item = items.front();
		items.pop_front();

		ROBOT* robot = NULL;

		int id = 0;

		char* action = strtok(item, " ");
		id = atoi(action);

		list<ROBOT*>::iterator robotIterator;
		for (robotIterator = robots.begin(); robotIterator!=robots.end(); ++robotIterator)
		{
			if ((*robotIterator)->id == id)
			{
				robot = *robotIterator;
				new_robots.push_back(robot);
				break;
			}
		}

		if (!robot)
		{
			robot = new ROBOT();
			robot->id = id;
			robot->packages = 0;
			robot->x = -1;
			robot->y = -1;
			new_robots.push_back(robot);
		}

		int moved = 0;
		int dropped = 0;

		action = strtok(NULL, " ");
		while (action)
		{
			switch(action[0])
			{
			case 'X':
				action = strtok(NULL, " ");
				robot->x = atoi(action);
				break;
			case 'Y':
				action = strtok(NULL, " ");
				robot->y = atoi(action);
				break;
			case 'P':
				action = strtok(NULL, " ");
				if (action)
				{
					int package_id = atoi(action);
					if (robot->id==my_id)
					{
						list<PACKAGE*>::iterator package_iterator;
						for (package_iterator=packages.begin(); package_iterator!=packages.end(); ++package_iterator)
							if ((*package_iterator)->id == package_id)
							{
								my_packages.push_back(*package_iterator);
								my_capacity -= (*package_iterator)->weight;

								packages.erase(package_iterator);

								break;
							}
					}

//					printf("Robot %i has picked-up package %i @ (%i,%i)\n", robot->id, package_id, robot->x, robot->y);

					robot->packages++;
				}
				break;
			case 'D':
				action = strtok(NULL, " ");
				if (action)
				{
					int package_id = atoi(action);
					if (robot->id == my_id)
					{
						list<PACKAGE*>::iterator package_iterator;
						for (package_iterator=my_packages.begin(); package_iterator!=my_packages.end(); ++package_iterator)
							if ((*package_iterator)->id == package_id)
							{
								if ((my_robot->x == (*package_iterator)->x) && (my_robot->y == (*package_iterator)->y))
									my_points += (*package_iterator)->weight;
								my_capacity += (*package_iterator)->weight;

								my_packages.erase(package_iterator);

								break;
							}
					}

//					printf("Robot %i has dropped package %i @ (%i,%i)\n", robot->id, package_id, robot->x, robot->y);

					robot->packages--;
				}
				dropped++;
				break;

			case 'E':
//				printf("Robot %i has gone East\n", id);
				robot->x++;
				moved++;

				if ((robot->id == my_robot->id) && (directions.size()))
				{
					if (directions.front() != 'E')
						robot->pushed = 1;
					directions.pop_front();
				}
				break;
			case 'W':
//				printf("Robot %i has gone west\n", id);
				robot->x--;
				moved++;

				if ((robot->id == my_robot->id) && (directions.size()))
				{
					if (directions.front() != 'W')
						robot->pushed = 1;
					directions.pop_front();
				}
				break;
			case 'S':
//				printf("Robot %i has gone South\n", id);
				robot->y--;
				moved++;
				if ((robot->id == my_robot->id) && (directions.size()))
				{
					if (directions.front() != 'S')
						robot->pushed = 1;
					directions.pop_front();
				}
				break;
			case 'N':
//				printf("Robot %i has gone North\n", id);
				robot->y++;
				moved++;
				if ((robot->id == my_robot->id) && (directions.size()))
				{
					if (directions.front() != 'N')
						robot->pushed = 1;
					directions.pop_front();
				}
				break;
			default:
				printf("Robot %i has done %c?!?!\n", id, action[0]);
				break;
			}
			action = strtok(NULL, " ");
		}

		if ((moved > 1) || (moved && dropped))
			robot->pushed = 1;
	}

	robots = new_robots;

//	for_each (robots.begin(), robots.end(), PrintRobot);
}

void CheckPackages(char* package_list)
{
 	printf("package_list: %s\n", package_list);

	while (packages.size())
	{
		PACKAGE* current = packages.front();
		packages.pop_front();
		delete current;
	}

	if (package_list)
	{
		char* package_id = strtok(package_list, " ");

		while (package_id)
		{
			char* package_x = strtok(NULL, " ");
			char* package_y = strtok(NULL, " ");
			char* package_weight = strtok(NULL, " ");

			PACKAGE* package = new PACKAGE;
			package->id = atoi(package_id);
			package->x = atoi(package_x);
			package->y = atoi(package_y);
			package->weight = atoi(package_weight);
			packages.push_back(package);

			package_id = strtok(NULL, " ");
		}
	}
	else
		printf("there are no packages to pickup\n");

	for_each (packages.begin(), packages.end(), PrintPackage);
}

void SolveMaze(int x1, int y1, int x2, int y2)
{
	printf("Trying to solve maze...from (%i,%i) to (%i,%i)\n", x1, y1, x2, y2);
	int width = game_board.width;
	int height = game_board.height;

	if (directions.size())
		directions.clear();

	int** tableau = new int*[height+1];
	int i;
	for (i = 0; i < height; i++)
		tableau[i] = new int[width+1];

	if (directions.size())
		directions.empty();

	if ((x1 == x2) && (y1 == y2))
		return;
	if (x1==-1 || x2==-1 || y1==-1 || y2==-1)
		return;

	int x, y;

	for (x = 0; x < width; x++)
	{
		for (y = 0; y < height; y++)
		{
			switch (game_board.value[y][x])
			{
			case '.':
				tableau[y][x] = -1;
				break;
			case '#':
				tableau[y][x] = -2;
				break;
			case '~':
				tableau[y][x] = -2;
				break;
			case '@':
				tableau[y][x] = -1;
				break;
			default:
				tableau[y][x] = 7;
			}
		}
	}

	tableau[y1-1][x1-1] = 0;
	tableau[y2-1][x2-1] = -10;

	// try to avoid an adjacent robot
	POSITION closest_robot = GetClosestRobot();
	if ((closest_robot.x+1 == my_robot->x) || (closest_robot.x-1 == my_robot->x) || (closest_robot.y+1 == my_robot->y) || (closest_robot.y-1 == my_robot->y))
		tableau[closest_robot.y-1][closest_robot.x-1] = -2;

	int found = 0;
	int iterations = 0;
	while (!found)
	{
		bool bugged = true;
		for (x = 0; x < width; x++)
			for (y = 0; y < height; y++)
				if (tableau[y][x] == iterations)
				{
					if ((x > 0) && (tableau[y][x-1]==-10))
						found = 1;
					if ((x < width-1) && (tableau[y][x+1] == -10))
						found = 1;
					if ((y > 0) && (tableau[y-1][x] == -10))
						found = 1;
					if ((y < height-1) && (tableau[y+1][x] == -10))
						found = 1;

					if (found)
						break;

					if ((x > 0) && (tableau[y][x-1]==-1))
					{
						bugged = false;
						tableau[y][x-1] = iterations+1;
					}
					if ((x < width-1) && (tableau[y][x+1] == -1))
					{
						bugged = false;
						tableau[y][x+1] = iterations+1;
					}
					if ((y > 0) && (tableau[y-1][x] == -1))
					{
						bugged = false;
						tableau[y-1][x] = iterations+1;
					}
					if ((y < height-1) && (tableau[y+1][x] == -1))
					{
						bugged = false;
						tableau[y+1][x] = iterations+1;
					}
				}
		iterations++;
		if (bugged && !found)
		{
			printf("Could not solve maze, waiting till next turn\n");
			return;
		}
	}

	int x_pos = x2-1;
	int y_pos = y2-1;
	for (int i = iterations-1; i >= 0; i--)
	{
		if ((y_pos < height-1) && (tableau[y_pos+1][x_pos] == i))
		{
			directions.push_front('S');
			y_pos++;
		}
		else if ((y_pos > 0) && (tableau[y_pos-1][x_pos] == i))
		{
			directions.push_front('N');
			y_pos--;
		}
		else if ((x_pos < width-1) && (tableau[y_pos][x_pos+1] == i))
		{
			directions.push_front('W');
			x_pos++;
		}
		else if ((x_pos > 0) && (tableau[y_pos][x_pos-1] == i))
		{
			directions.push_front('E');
			x_pos--;
		}
	}

	printf("path to go from (%i,%i) to (%i,%i) in %i steps:\n", x1,y1,x2,y2, iterations);
	for (list<char>::iterator iterator=directions.begin(); iterator != directions.end(); ++iterator)
		printf("%c", *iterator);
	printf("\n");
}

void MakeUpYourMind()
{
	char* command = NULL;
	int wager = 3;
	list<PACKAGE*> pickup_packages;
	list<PACKAGE*>::iterator iterator;
	POSITION my_position = {my_robot->x,my_robot->y};
	POSITION closest_robot = GetClosestRobot();

	if (my_robot->pushed)
	{
		if (directions.size())
			directions.clear();
		my_robot->pushed = 0;
		printf("I was just pushed\n");
	}

	if (Distance(my_position, closest_robot) == 1)
	{


	}

	// s'il-y-a des paquets on dresse une liste de tous ceux que l'on peut prendre.
	if (packages.size())
	{
		printf("are there packages i can pickup?\n");
		int capacity = my_capacity;
		for (iterator= packages.begin(); iterator != packages.end(); ++iterator)
		{
			if ((*iterator)->weight <= capacity)
			{
				pickup_packages.push_back(*iterator);
				capacity -= (*iterator)->weight;
			}
		}
	} // s'il n'y-a pas de paquets et que c'est une homebase, on la transforme en '.' pour ne pas y revenir
	else if (game_board.value[my_robot->y-1][my_robot->x-1] == '@')
	{
		printf("this homebase is empty...\n");
		game_board.value[my_robot->y-1][my_robot->x-1] = '.';
	}


	// if there are packages to be dropped, we drop them.
	if (SomethingToDrop())
	{
		printf("i have pakages to drop here\n");
		command = new char[(1+sizeof(int))*my_packages.size()+6];
		sprintf(command, "%i Drop", wager);
		if (my_packages.size())
			for (iterator= my_packages.begin(); iterator != my_packages.end(); ++iterator)
				if (((*iterator)->x == my_robot->x) && ((*iterator)->y == my_robot->y))
					sprintf(command, "%s %i", command, (*iterator)->id);

		directions.clear();
	} // otherwise, if there are packaes to pickup, we pick them up
	else if (pickup_packages.size())
	{
		printf("i have packages to pickup here\n");
		command = new char[(1+sizeof(int))*pickup_packages.size()+6];
		memset(command, 0, sizeof((1+sizeof(int))*pickup_packages.size()+6));
		sprintf(command, "%i Pick", wager);
		if (pickup_packages.size())
			for (iterator= pickup_packages.begin(); iterator != pickup_packages.end(); ++iterator)
				sprintf(command, "%s %i", command, (*iterator)->id);

		pickup_packages.clear();
		directions.clear();
	} // if we have directions, we follow them
	else if (directions.size())
	{
		char direction = directions.front();

		printf("continuing current direction (%c)/%i\n", direction, directions.size());

		command = new char[50];
		sprintf(command, "%i Move %c", wager, direction);
//		directions.pop_front();
	} // otherwise we find somewhere to go
	else
	{
		printf("where should i go?\n");
		POSITION base = GetClosestBase();
		POSITION robot = GetClosestPackageRobot();
		POSITION closest_package = GetClosestPackageDropZone();
		POSITION heaviest_package = GetHeaviestPackageDropZone();
		POSITION my_position = {my_robot->x, my_robot->y};

		printf("The Closest homebase is (%i,%i) @ %i\n", base.x, base.y,Distance(my_position, base));
		printf("The Closest robot is (%i,%i) @ %i\n", robot.x, robot.y,Distance(my_position, robot));
		printf("The Closest package is (%i,%i) @ %i\n", closest_package.x, closest_package.y, Distance(my_position, closest_package));
		printf("The Heaviest package is (%i,%i) @ %i\n", heaviest_package.x, heaviest_package.y,Distance(my_position, heaviest_package));

		if (closest_package.x != -1)
			my_destination = closest_package;
//		if (heaviest_package.x != -1)
//			my_destination = heaviest_package;
		else if (base.x != -1 && robot.x != -1)
			my_destination = (Distance(my_position, base) > Distance(my_position, robot)) ? base : robot;
		else if (base.x != -1)
			my_destination = base;
		else if (robot.x != -1)
			my_destination = robot;

		printf("I'm heading towards (%i,%i)\n", my_destination.x, my_destination.y);
		//getchar();

		if (Distance(my_position, my_destination) == 1)
		{
			if (directions.size())
				directions.clear();

			if (my_destination.y < my_position.y)
				directions.push_front('S');
			else if (my_destination.y > my_position.y)
				directions.push_front('N');
			else if (my_destination.x < my_position.x)
				directions.push_front('W');
			else
				directions.push_front('E');

		}
		else
			SolveMaze(my_robot->x, my_robot->y, my_destination.x, my_destination.y);

		command = new char[50];
		if (directions.size())
		{
			sprintf(command, "%i Move %c", wager, directions.front());
	//		directions.pop_front();
		}
		else
			sprintf(command, "%i Pick", wager);
	}

	printf("Sending command (%s)...\n", command);
	sprintf(command, "%s\n", command);

	send(server_socket, command, strlen(command), 0);

	my_money -= wager;

	delete[] command;
}

POSITION GetClosestBase()
{
	POSITION closest = {-1, -1};
	int min_distance = game_board.width + game_board.height + 1;

	for (int x = 0; x < game_board.width; x++)
		for (int y = 0; y < game_board.height; y++)
			if (game_board.value[y][x] == '@')
			{
				int distance = abs(x-my_robot->x) + abs(y-my_robot->y);
				if (distance < min_distance)
				{
					min_distance = distance;
					closest.x = x+1;
					closest.y = y+1;
				}
			}

	return closest;
}


POSITION GetClosestRobot()
{
	POSITION closest = {-1, -1};
	int min_distance = game_board.width + game_board.height + 1;
	list<ROBOT*>::iterator iterator;

	if (robots.size())
		for (iterator=robots.begin(); iterator != robots.end(); ++iterator)
		{
			if ((*iterator)->id != my_robot->id)
			{
				int distance = abs((*iterator)->x-my_robot->x) + abs((*iterator)->y-my_robot->y);
				if (distance < min_distance)
				{
					min_distance = distance;
					closest.x = (*iterator)->x;
					closest.y = (*iterator)->y;
				}
			}
		}

	return closest;
}

POSITION GetClosestPackageRobot()
{
	POSITION closest = {-1, -1};
	int min_distance = game_board.width + game_board.height + 1;
	list<ROBOT*>::iterator iterator;

	if (robots.size())
		for (iterator=robots.begin(); iterator != robots.end(); ++iterator)
		{
			if (((*iterator)->id != my_robot->id) && ((*iterator)->packages))
			{
				int distance = abs((*iterator)->x-my_robot->x) + abs((*iterator)->y-my_robot->y);
				if (distance < min_distance)
				{
					min_distance = distance;
					closest.x = (*iterator)->x;
					closest.y = (*iterator)->y;
				}
			}
		}

	return closest;
}

POSITION GetClosestPackageDropZone()
{
	POSITION closest = {-1, -1};
	int min_distance = game_board.width + game_board.height + 1;
	list<PACKAGE*>::iterator iterator;

	if (my_packages.size())
		for (iterator=my_packages.begin(); iterator != my_packages.end(); iterator++)
		{
			int distance = abs((*iterator)->x-my_robot->x) + abs((*iterator)->y-my_robot->y);
			if (distance < min_distance)
			{
				min_distance = distance;
				closest.x = (*iterator)->x;
				closest.y = (*iterator)->y;
			}

		}

	return closest;
}

POSITION GetHeaviestPackageDropZone()
{
	POSITION closest = {-1, -1};
	int max_weight = 0;
	list<PACKAGE*>::iterator iterator;

	if (my_packages.size())
		for (iterator=my_packages.begin(); iterator != my_packages.end(); iterator++)
		{
			if ((*iterator)->weight > max_weight)
			{
				max_weight = (*iterator)->weight;
				closest.x = (*iterator)->x;
				closest.y = (*iterator)->y;
			}

		}

	return closest;
}

bool SomethingToDrop()
{
	list<PACKAGE*>::iterator iterator;

	if (my_packages.size())
		for (iterator= my_packages.begin(); iterator != my_packages.end(); ++iterator)
			if (((*iterator)->x == my_robot->x) && ((*iterator)->y == my_robot->y))
				return true;

	return false;
}

int NumberOfWalls(POSITION position)
{
	int result = 0;
	if (game_board.value[position.y-1][position.x] == '#')
		result++;
	if (game_board.value[position.y-1][position.x-2] == '#')
		result++;
	if (game_board.value[position.y][position.x-1] == '#')
		result++;
	if (game_board.value[position.y-2][position.x-1] == '#')
		result++;

	return result;
}

int NumberOfWater(POSITION position)
{
	int result = 0;
	if (game_board.value[position.y-1][position.x] == '~')
		result++;
	if (game_board.value[position.y-1][position.x-2] == '~')
		result++;
	if (game_board.value[position.y][position.x-1] == '~')
		result++;
	if (game_board.value[position.y-2][position.x-1] == '~')
		result++;

	return result;
}